#!/usr/bin/perl
use Crypt::Blowfish;
use Digest::MD5 qw/ md5_hex /;

my $path = "/usr/local/apache/htdocs/audio";
my $pathurl = "/audio";

my $vmroot = "/var/spool/asterisk/monitor";
my $string = $ENV{QUERY_STRING};
$string =~ s/\.wav//g;
my ($vm_format) = $ENV{QUERY_STRING} =~ m/vm_format=(\w+)/;
$string =~ s/\&vm_format=\w+//g;

open(KEY, "</etc/asterisk/key") || die "can't open key file: $!";
chomp(my $key = <KEY>);
close(KEY);

$key = pack("H78", $key);
my $cipher = new Crypt::Blowfish $key;

my ($text, $server_id);
my @blocks = unpack("a16" x (int(length($string) / 8) + 1), $string);
foreach my $block (@blocks)
{
	if($block eq '')
	{
		last;
	}
	$text .= $cipher->decrypt(pack("H16", $block));
	$text =~ s/[^\+\w\s\-\,\.\/\:\?\`]//g;
}

# trim the trailing whitespaces
$text =~ s/\s*$//g;

$text =~ s/\.([^.]+?)\s*$/\.$1/;
($text, $server_id) = split /`/, $text;

my $file = $server_id ? "/data/$server_id$vmroot/$text" : "$vmroot/$text";
my $ext = $text;
$ext =~ s/^.*\.(\w+?)$/$1/;

if (-e $file && $text ne "")
{
	if ($vm_format eq 'mp3') {
		if (!md5path_mp3($file)) {
			print "Content-Type: text/html\n\n";
			print "<b>Error Message:</b> Could not playback file: $file<br>\n<b>Error Type:</b> MD5 Path Creation\n";
		}
		else {
			print "Location: " . md5path_mp3($file, $path, $pathurl) . "\r\n\r\n";
		}
	} else {
		print "Location: " . randpath($file, $ext, $path, $pathurl) . "\r\n\r\n";
	}
}
else
{
	print "Content-Type: text/html\n\n";
	print "File not found ^$file^\n";
}

sub randpath
{
	my $file = shift;
	my $ext = shift;
	my $path = shift;
	my $pathurl = shift;

	my $rand = random_characters();
	link($file, "$path/$rand.$ext");
	chmod(0755, "$path/$rand.$ext");

	return "$pathurl/$rand.$ext";
}

=pod

Create an .mp3 from a .wav and return its location.

        Args: path of wav voicemail file
        Returns: URL to an mp3 version of the voicemail or undef

=cut

sub md5path_mp3
{
	my $wav = shift;
	my $path = shift;
	my $pathurl = shift;

	my $md5_hex;
	return if !($md5_hex = md5_hex_head($wav));
	my $wav_tmp = "$path/${md5_hex}.wav";
	if (! -e  $wav_tmp) {
		link($wav, $wav_tmp);
		chmod(0755, $wav_tmp);
	}
	if (! -e  "${wav_tmp}.mp3") {
		# Resample encoding to 16 kHz for better voice quality.
		system("lame --quiet --resample 16 ${wav_tmp}");
	}
	return "$pathurl/${md5_hex}.wav.mp3";
}

# Create a random string of characters with default length of 12.
sub random_characters {
	my $number_of_characters = shift || 12;

	my @chars = ("a".."z","A".."Z",0..9);
	my $rand;
	foreach (1 .. $number_of_characters)
	{
		$rand .= $chars[int(rand(@chars))];
	}

	return $rand;
}

=pod

 Compute md5 on the first 300kb of file
 or on the whole file when the file size is < 300kb.

 Args: $file_name
 Returns: md5_hex of first 300kb of file on success otherwise false

=cut

sub md5_hex_head
{
	my $file = shift;

	my $length = 307200;
	my $buffer = '';
	open(F, '<', $file) || return;
	my $bytes_read = sysread(F, $buffer, $length);
	close F;

	return md5_hex($buffer);
}

